@@ -95,7 +95,7 @@ def get_mission_location_table() -> typing.List[LocationData]:
             for j in range(50):
                 location_name = f"{location.name} Outlaw {j + 1}"
                 location_id = location.id + mission_start_id + (100 * location.id) + j + 50
-                new_list.append(LocationData("Mission", 0, location_name, location_id, 0, []))
+                new_list.append(LocationData("Outlaw", 0, location_name, location_id, 0, []))
 
     return new_list
 
@@ -232,7 +232,7 @@ EOS_location_table: typing.List[LocationData] = [
     LocationData("SpecialDungeonComplete", 2, "SE Middle Spring Cave", 161, 160, ["Special"]),  # 7 subareas
     LocationData("SpecialDungeonComplete", 3, "SE Spring Cave Pit", 164, 162, ["Special"]),  # 7 subareas
 
-    LocationData("EarlyDungeonComplete", 1, "Star Cave", 174,  174, ["Mission", "Late"]),
+    LocationData("EarlyDungeonComplete", 1, "Star Cave", 174,  174, ["Mission", "Early"]),
     # Dojo Dungeons
     LocationData("DojoDungeonComplete", 1, "Dojo Normal/Fly Maze", 180, 180, ["Dojo"]),  # 7 subareas
     LocationData("DojoDungeonComplete", 1, "Dojo Dark/Fire Maze", 181, 181, ["Dojo"]),  # 7 subareas
